package cn.iselab.mooctest.site.service.impl;

import cn.iselab.mooctest.site.service.AutoReportProductService;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import java.io.*;

@Service
public class AutoReportProductServiceImpl implements AutoReportProductService {

    private static final Logger log = LoggerFactory.getLogger(Process.class.getName());

    @Value("${order.autoReport.service.path}")
    private String REPORT_SERVICE_PATH;
    @Value("${order.autoReport.upload.path}")
    private String REPORT_UPLOAD_PATH;
    @Value("${order.autoReport.image.upload.path}")
    private String REPORT_IMAGE_UPLOAD_PATH;
    @Value("${order.autoReport.builder.copyPath}")
    private String BUILDER_COPYPATH;
    @Value("${static.resource.autoReport.url}")
    private String AUTOREPORT_URL;

    @Override
    public void writeDataToJsonFile(String jsonData, String path) {
        File file = new File(path, "reportData.json");
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
        }
        if (file.exists()) {
            file.delete();
        }
        try {
            file.createNewFile();
            FileUtils.writeStringToFile(file, jsonData);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public boolean runBuilderToProductReport(String command, String path) {
        File file = new File(path);
        try {
            log.info("-----------file exist:{}-------",file.exists());
            Process process = Runtime.getRuntime().exec(command, null, file);
            new Thread(){
                @Override
                public void run(){
                    BufferedReader in = new BufferedReader(new InputStreamReader(process.getInputStream()));
                    BufferedReader err = new BufferedReader(new InputStreamReader(process.getErrorStream()));
                    String line = null;
                    String errline = null;
                    try {
                        while ((line = in.readLine())!=null || (errline = err.readLine())!=null){
                            if(line!=null){
                                log.info(line);
                            }
                            if(errline!=null) {
                               log.info(errline);
                            }
                        }
                    }catch (IOException e){
                        log.error("-------generateReport readInputstreamException:{}",e);
                    }finally {
                        try {
                            in.close();
                            err.close();
                        }catch (IOException e){
                            log.error("-------generateReport closeInputstreamException:{}",e);
                        }
                    }
                }
            }.start();

            int status = process.waitFor();
            if (status == 0) {
                System.out.println(status);
                return true;
            }
            return false;
        } catch (Exception e) {
            log.error("-------------npm run error{}-----------",e);
            return false;
        }
    }


    @Override
    public String copyReportBuilderToPath(Long orderId, String reportBuilderName) {
        File file = new File(BUILDER_COPYPATH + orderId);
        File reportFile = new File(REPORT_SERVICE_PATH);
        String command = "cp -r " + reportBuilderName + " " + BUILDER_COPYPATH + orderId + "/";
        if (!file.exists()) {
            log.info("-------------开始拷贝生成器---------------");
            file.mkdirs();
            try {
                Process process = Runtime.getRuntime().exec(command, null, reportFile);
                new Thread() {
                    @Override
                    public void run() {
                        BufferedReader in = new BufferedReader(new InputStreamReader(process.getInputStream()));
                        BufferedReader err = new BufferedReader(new InputStreamReader(process.getErrorStream()));
                        String line = null;
                        String errLine = null;
                        try {
                            while ((line=in.readLine())!=null||(errLine=err.readLine())!=null) {
                                if(line!=null)
                                    log.info(line);
                                if(errLine!=null)
                                    log.info(errLine);
                            }
                        } catch (IOException e) {
                            log.error("-------copyReportBuilder readInputstreamException:{}",e);
                        } finally {
                            try {
                                in.close();
                                err.close();
                            }catch (IOException e){
                                log.error("-------copyReportBuilder closeInputstreamException:{}",e);
                            }
                        }
                    }
                }.start();
                process.waitFor();
            } catch (Exception e) {
                log.info("-------------copy reportBuild error{}--------------",e);
            }
        }
        return BUILDER_COPYPATH + orderId;
    }
}
